'use strict';

var _mocha = require('mocha');

var _mocha2 = _interopRequireDefault(_mocha);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _testsuite = require('./testsuite');

var _testsuite2 = _interopRequireDefault(_testsuite);

var _test2 = require('./test');

var _test3 = _interopRequireDefault(_test2);

var _streams = require('./streams');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const REPORT_FILE = process.env.REPORT_FILE; /* eslint-disable
                                              no-param-reassign,
                                              */

const install = reportFile => {
  const name = _path2.default.basename(reportFile, _path2.default.extname(reportFile)).replace(/\//g, '.');
  let testsuite = null;
  let nonTestFailed = false;

  function _test(test) {
    if (test._junitTest == null) {
      test._junitTest = new _test3.default(test, testsuite);
      testsuite.addTest(test._junitTest);
    }
    return test._junitTest;
  }

  class MyRunner extends _mocha2.default.Runner {
    constructor(...args) {
      super(...args);
      testsuite = new _testsuite2.default(name);

      this.on('test', test => {
        test._junitCaptureTest = _test(test);
      });

      this.on('fail', (failed, err) => {
        const fail = test => _test(test).fail(failed, err);
        let test = failed;
        if (failed.type === 'hook') {
          if (failed.ctx.currentTest) {
            fail(failed.ctx.currentTest);
          } else if (String(failed.title).startsWith('"before all" hook')) {
            let first = null;
            failed.parent.eachTest(t => {
              if (!first) first = t;
            });
            fail(first);
          } else if (String(failed.title).startsWith('"after all" hook')) {
            let last = null;
            failed.parent.eachTest(t => {
              last = t;
            });
            fail(last);
          } else {
            failed.parent.eachTest(fail);
          }
        } else if (failed.type === 'test') {
          test = failed.ctx && failed.ctx.currentTest || failed;
          fail(test);
        } else {
          nonTestFailed = true;
        }
      });

      this.on('pass', test => {
        _test(test).pass();
      });

      this.on('hook', hook => {
        if (hook.ctx.currentTest != null) {
          const test = _test(hook.ctx.currentTest);
          hook._streamsCaptured = true;
          (0, _streams.captureStreams)(test);
        }
      });

      this.on('hook end', hook => {
        if (hook._streamsCaptured) {
          delete hook._streamsCaptured;
          (0, _streams.releaseStreams)();
        }
      });
    }

    run(fn) {
      console.log('Running mocha-junit, will write results to %s', reportFile);
      const suite = this.suite;
      suite.eachTest(test => _test(test));
      super.run(() => {
        testsuite.end();
        testsuite.writeFile(reportFile, err => {
          if (err != null) {
            throw err;
          }
          return fn(nonTestFailed ? 1 : 0);
        });
      });
    }

    runTest(fn) {
      const test = this.test._junitCaptureTest;
      if (test != null) {
        delete this.test._junitCaptureTest;
        fn = (0, _streams.withReleaseStreams)(fn);
        (0, _streams.captureStreams)(test);
      }
      super.runTest(fn);
    }
  }

  _mocha2.default.Runner = MyRunner;
};

if (REPORT_FILE) {
  install(REPORT_FILE);
}