'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _hasStreams = require('./has-streams');

var _hasStreams2 = _interopRequireDefault(_hasStreams);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function fullTitle(runnable) {
  const result = [];
  let r = runnable;
  while (r != null) {
    if (r.title !== '') {
      result.push(r.title);
    }
    r = r.parent;
  }
  return result.reverse().join('.');
}

class Test extends _hasStreams2.default {
  constructor(test, suite) {
    super();
    this.test = test;
    this.failures = [];
    this.failed = false;
    this.passed = false;
    this.suite = suite;
  }

  fail(failed, err) {
    this.failed = true;
    this.passed = false;
    let message = err && err.message ? err.message : 'unknown error';
    if (failed !== this.test) {
      message += ` (from: ${fullTitle(failed)})`;
    }
    let content = '';
    if (err && err.stack) {
      content = err.stack.replace(/^/gm, '  ');
    }
    return this.failures.push({ message, content });
  }

  pass() {
    this.passed = true;
  }

  write(writable) {
    const skipped = !(this.passed || this.failed);
    const attrs = {
      classname: `${this.suite.name}.${fullTitle(this.test.parent)}`,
      name: this.test.title
    };
    if (!skipped) {
      attrs.time = this.test.duration / 1000;
    }
    this.openTag(writable, 'testcase', attrs);
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = this.failures[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const failure = _step.value;

        this.openTag(writable, 'failure', { message: failure.message });
        this.escape(writable, failure.content);
        this.closeTag(writable, 'failure');
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    if (skipped) {
      this.emptyTag(writable, 'skipped');
    }
    super.write(writable);
    return this.closeTag(writable, 'testcase');
  }
}
exports.default = Test;