'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _hasStreams = require('./has-streams');

var _hasStreams2 = _interopRequireDefault(_hasStreams);

var _testsuites = require('./testsuites');

var _testsuites2 = _interopRequireDefault(_testsuites);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Testsuite extends _hasStreams2.default {
  constructor(name, parent) {
    super();
    this.name = name;
    this.parent = parent;
    this.tests = [];
    this.failures = 0;
    this.errors = 0;
    this.timestamp = new Date();
    this.hostname = _os2.default.hostname();
    this.properties = {};
    if (this.parent == null) {
      this.parent = new _testsuites2.default(this.name);
    }
    this.parent.suites.push(this);
  }

  addTest(test) {
    this.tests.push(test);
  }

  end() {
    this.time = new Date() - this.timestamp;
  }

  write(writable) {
    let failures = 0;
    let passed = 0;
    const total = this.tests.length;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = this.tests[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const test = _step.value;

        if (test.failed) {
          failures += 1;
        }
        if (test.passed) {
          passed += 1;
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    this.openTag(writable, 'testsuite', {
      name: this.name,
      hostname: this.hostname,
      timestamp: this.timestamp.toUTCString(),
      time: this.time / 1000,
      tests: total,
      failures: this.failures + failures,
      skipped: total - failures - passed,
      errors: this.errors
    });

    const keys = Object.keys(this.properties);
    if (keys.length) {
      this.openTag(writable, 'properties');
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = keys[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          const name = _step2.value;

          const value = this.properties[name];
          this.emptyTag(writable, 'property', { name, value });
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      this.closeTag(writable, 'properties');
    }

    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
      for (var _iterator3 = this.tests[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
        const test = _step3.value;

        test.write(writable);
      }
    } catch (err) {
      _didIteratorError3 = true;
      _iteratorError3 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion3 && _iterator3.return) {
          _iterator3.return();
        }
      } finally {
        if (_didIteratorError3) {
          throw _iteratorError3;
        }
      }
    }

    super.write(writable);
    this.closeTag(writable, 'testsuite');
  }
}
exports.default = Testsuite;