'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function buildEscapes() {
  const escapes = {
    '&': '&amp;',
    "'": '&apos;',
    '"': '&quot;',
    '<': '&lt;',
    '>': '&gt;'
  };

  const regex = new RegExp(`[${Object.keys(escapes).join('')}\\x00-\\x1f]`, 'g');
  for (let i = 0; i < 32; i += 1) {
    escapes[String.fromCharCode(i)] = `&#x${i.toString(16)};`;
  }
  escapes['\t'] = '\t';
  escapes['\r'] = '\r';
  escapes['\n'] = '\n';
  return str => String(str).replace(regex, x => escapes[x]);
} /* eslint-disable class-methods-use-this */


const _xmlEscape = buildEscapes();

class XMLWriter {
  _startTag(writable, name, attrs = {}) {
    writable.write(`<${name}`);
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = Object.keys(attrs)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const k = _step.value;

        const v = attrs[k];
        if (v != null) {
          writable.write(` ${k}="`);
          this.escape(writable, v);
          writable.write('"');
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return writable;
  }

  openTag(writable, name, attrs) {
    return this._startTag(writable, name, attrs).write('>\n');
  }

  emptyTag(writable, name, attrs) {
    return this._startTag(writable, name, attrs).write('/>\n');
  }

  closeTag(writable, name) {
    return writable.write(`</${name}>\n`);
  }

  escape(writable, s) {
    return writable.write(_xmlEscape(s));
  }

  writeFile(output, cb) {
    // Create directory if it doesn't exist
    return (0, _mkdirp2.default)(_path2.default.dirname(output), err => {
      if (err != null) {
        return cb(err);
      }
      const writable = _fs2.default.createWriteStream(output);
      writable.write('<?xml version="1.1"?>\n');
      this.write(writable);
      return writable.end(cb);
    });
  }
}
exports.default = XMLWriter;